package de.ozgcloud.eingang.common.formdata;

import java.util.EnumSet;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import lombok.AccessLevel;
import lombok.Builder;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Singular;

@Getter
@Builder
public class ServiceKonto {

	private String type;
	@Singular
	private List<PostfachAddress> postfachAddresses;
	private String trustLevel;

	@Getter
	@Builder
	public static class PostfachAddress {
		private int type;
		private String version;
		private PostfachAddressIdentifier identifier;
	}

	@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
	public enum TrustLevel {

		LEVEL_1("STORK-QAA-Level-1"),
		LEVEL_2("STORK-QAA-Level-2"),
		LEVEL_3("STORK-QAA-Level-3"),
		LEVEL_4("STORK-QAA-Level-4");

		private final String value;

		public static boolean exists(String trustLevelValue) {
			if (StringUtils.isBlank(trustLevelValue)) {
				return false;
			}
			return EnumSet.allOf(TrustLevel.class).stream().map(trustLevel -> trustLevel.value).anyMatch(trustLevelValue::equalsIgnoreCase);
		}
	}

}