package de.ozgcloud.eingang.common.vorgang;

import java.time.LocalDate;

import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
public class VorgangNummerSupplier {

	static final String VORGANGNUMMER_TEMPLATE = "%d%X%02d-%s";
	static final char[] BASE30_ALPHABET = { '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'M',
			'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'X', 'Y', 'Z' };
	static final int SUFFIX_LENGTH = 6;

	public String get() {
		return get(SUFFIX_LENGTH);
	}

	public String get(int suffixLength) {
		if (suffixLength <1){
			throw new TechnicalException("Suffix length must be at least 1");
		}
		var today = LocalDate.now();
		var lastYearNumber = today.getYear() % 10;
		return VORGANGNUMMER_TEMPLATE.formatted(lastYearNumber, today.getMonthValue(), today.getDayOfMonth(),
				RandomStringUtils.random(suffixLength, BASE30_ALPHABET));
	}

}
